import { app, ipcMain, IpcMessageEvent, BrowserWindow } from 'electron';
import * as path from 'path';
import * as url from 'url';
import * as fse from 'fs-extra';
import * as log from 'electron-log';

import { now } from './util/performance';
import { showUncaughtException } from './crash/crash';
import { withSourceMappedStack } from './util/source-map-support';
import { reportError } from './util/report';

const launchStartTime = now();
let launchReadyTime: number | null = null;

let preventQuit = false;

const appRoot = path.join(__dirname, `../../dist/odds-crawler-frontend-app`);
process.env['ELECTRON_DISABLE_SECURITY_WARNINGS'] = 'true';

let mainWindow: BrowserWindow | null = null;

function handleUncaughtException(error: Error) {
  preventQuit = true;

  const isLaunchError = mainWindow === null;

  if (mainWindow) {
    mainWindow.destroy();
    mainWindow = null;
  }

  showUncaughtException(isLaunchError, error);
}
process.on('uncaughtException', (error: Error) => {
  error = withSourceMappedStack(error);

  reportError(error);
  handleUncaughtException(error);
});

let isDuplicateInstance = false;
const gotSingleInstanceLock = app.requestSingleInstanceLock();
isDuplicateInstance = !gotSingleInstanceLock;

app.on('second-instance', (event, args, workingDirectory) => {
  // Someone tried to run a second instance, we should focus our window.
  if (mainWindow) {
    if (mainWindow.isMinimized()) {
      mainWindow.restore();
    }

    if (!mainWindow.isVisible()) {
      mainWindow.show();
    }

    mainWindow.focus();
  }
});

if (isDuplicateInstance) {
  app.quit();
}

app.on('ready', () => {
  if (isDuplicateInstance) {
    return;
  }

  launchReadyTime = now() - launchStartTime;

  createWindow();

  ipcMain.on(
    'uncaught-exception',
    (event: Electron.IpcMessageEvent, error: Error) => {
      handleUncaughtException(error);
    }
  );
});

let windowStateKeeper: any | null = null;

function createWindow() {
  const minWidth = 960;
  const minHeight = 660;

  if (!windowStateKeeper) {
    // `electron-window-state` requires Electron's `screen` module, which can
    // only be required after the app has emitted `ready`. So require it
    // lazily.
    windowStateKeeper = require('electron-window-state');
  }

  const savedWindowState = windowStateKeeper({
    defaultWidth: minWidth,
    defaultHeight: minHeight
  });

  const windowOptions: Electron.BrowserWindowConstructorOptions = {
    x: savedWindowState.x,
    y: savedWindowState.y,
    width: savedWindowState.width,
    height: savedWindowState.height,
    minWidth: minWidth,
    minHeight: minHeight,
    show: false,
    // This fixes subpixel aliasing on Windows
    // See https://github.com/atom/atom/commit/683bef5b9d133cb194b476938c77cc07fd05b972
    backgroundColor: '#fff',
    webPreferences: {
      // Disable auxclick event
      // See https://developers.google.com/web/updates/2016/10/auxclick
      disableBlinkFeatures: 'Auxclick',
      // Enable, among other things, the ResizeObserver
      experimentalFeatures: true,
      nodeIntegration: true
    },
    acceptFirstMouse: true
  };

  if (__DARWIN__) {
    windowOptions.titleBarStyle = 'hidden';
  } else if (__WIN32__) {
    windowOptions.frame = false;
  } else if (__LINUX__) {
    windowOptions.icon = path.join(__dirname, 'static', 'icon-logo.png');
  }

  mainWindow = new BrowserWindow(windowOptions);
  savedWindowState.manage(mainWindow);

  let quitting = false;
  app.on('before-quit', () => {
    quitting = true;
  });

  ipcMain.on('will-quit', (event: Electron.IpcMessageEvent) => {
    quitting = true;
    event.returnValue = true;
  });

  let startLoad = 0;
  // We only listen for the first of the loading events to avoid a bug in
  // Electron/Chromium where they can sometimes fire more than once. See
  // See
  // https://github.com/desktop/desktop/pull/513#issuecomment-253028277. This
  // shouldn't really matter as in production builds loading _should_ only
  // happen once.
  mainWindow.webContents.once('did-start-loading', () => {
    // this._rendererReadyTime = null;
    // this._loadTime = null;

    startLoad = now();
  });

  mainWindow.webContents.once('did-finish-load', () => {
    if (process.env.NODE_ENV === 'development') {
      mainWindow.webContents.openDevTools();
    }

    // this._loadTime = now() - startLoad;

    // this.maybeEmitDidLoad();
  });

  mainWindow.webContents.on('did-finish-load', () => {
    mainWindow.webContents.setVisualZoomLevelLimits(1, 1);
  });

  mainWindow.webContents.on('did-fail-load', () => {
    mainWindow.webContents.openDevTools();
    mainWindow.show();
  });

  // TODO: This should be scoped by the window.
  ipcMain.once(
    'renderer-ready',
    (event: Electron.IpcMessageEvent, readyTime: number) => {
      // this._rendererReadyTime = readyTime;
      // this.maybeEmitDidLoad();
    }
  );

  mainWindow.loadURL(
    url.format({
      pathname: path.join(appRoot, 'index.html'),
      protocol: 'file:',
      slashes: true
    })
  );

  mainWindow.once('ready-to-show', () => {
    mainWindow.show();
  });

  mainWindow.on('closed', function() {
    mainWindow = null;
  });
}

app.on('window-all-closed', function() {
  if (process.platform !== 'darwin') {
    app.quit();
  }
});

app.on('activate', function() {
  if (mainWindow === null) {
    createWindow();
  }
});
